@@ -18,6 +18,7 @@
  **********************************************************************************/
 
 #include "kemoticons.h"
+#include "kemoticonsglobal_p.h"
 #include "kemoticonsprovider.h"
 
 #include <QFile>
@@ -28,24 +29,30 @@
 #include <QFileSystemWatcher>
 
 #include <kpluginloader.h>
+#include <kpluginmetadata.h>
 #include <kconfiggroup.h>
 #include <ksharedconfig.h>
 #include <ktar.h>
 #include <kzip.h>
 
+Q_GLOBAL_STATIC(KEmoticonsGlobal, s_global)
+
 class KEmoticonsPrivate
 {
 public:
     KEmoticonsPrivate(KEmoticons *parent);
     ~KEmoticonsPrivate();
     void loadServiceList();
+    KEmoticonsProvider *loadProvider(const KPluginMetaData &plugin);
     KEmoticonsProvider *loadProvider(const KService::Ptr &service);
     KEmoticonsTheme loadTheme(const QString &name);
 
-    QList<KService::Ptr> m_loaded;
+    QList<KService::Ptr> m_oldStylePlugins;
+    QVector<KPluginMetaData> m_plugins;
     QHash<QString, KEmoticonsTheme> m_themes;
     QFileSystemWatcher m_fileWatcher;
     KEmoticons *q;
+    QSize m_preferredSize;
 
     //private slots
     void changeTheme(const QString &path);
@@ -60,16 +67,32 @@ KEmoticonsPrivate::~KEmoticonsPrivate()
 {
 }
 
-bool priorityLessThan(const KService::Ptr &s1, const KService::Ptr &s2)
+static bool priorityLessThan(const KService::Ptr &s1, const KService::Ptr &s2)
 {
     return (s1->property(QStringLiteral("X-KDE-Priority")).toInt() > s2->property(QStringLiteral("X-KDE-Priority")).toInt());
 }
 
 void KEmoticonsPrivate::loadServiceList()
 {
-    QString constraint("(exist Library)");
-    m_loaded = KServiceTypeTrader::self()->query(QStringLiteral("KEmoticons"), constraint);
-    qSort(m_loaded.begin(), m_loaded.end(), priorityLessThan);
+    const QString constraint(QStringLiteral("(exist Library)"));
+    m_oldStylePlugins = KServiceTypeTrader::self()->query(QStringLiteral("KEmoticons"), constraint);
+    qSort(m_oldStylePlugins.begin(), m_oldStylePlugins.end(), priorityLessThan);
+
+    m_plugins = KPluginLoader::findPlugins(QStringLiteral("kf5/emoticonsthemes"));
+    std::sort(m_plugins.begin(), m_plugins.end(), [](const KPluginMetaData &s1, const KPluginMetaData &s2) {
+            return s1.rawData().value(QStringLiteral("X-KDE-Priority")).toInt() > s2.rawData().value(QStringLiteral("X-KDE-Priority")).toInt();
+    });
+}
+
+KEmoticonsProvider *KEmoticonsPrivate::loadProvider(const KPluginMetaData &plugin)
+{
+    KPluginFactory *factory = qobject_cast<KPluginFactory *>(plugin.instantiate());
+    if (!factory) {
+        qWarning() << "Invalid plugin factory for" << plugin.fileName();
+        return nullptr;
+    }
+    KEmoticonsProvider *provider = factory->create<KEmoticonsProvider>(nullptr);
+    return provider;
 }
 
 KEmoticonsProvider *KEmoticonsPrivate::loadProvider(const KService::Ptr &service)
@@ -77,16 +100,16 @@ KEmoticonsProvider *KEmoticonsPrivate::loadProvider(const KService::Ptr &service
     KPluginFactory *factory = KPluginLoader(service->library()).factory();
     if (!factory) {
         qWarning() << "Invalid plugin factory for" << service->library();
-        return 0;
+        return nullptr;
     }
-    KEmoticonsProvider *provider = factory->create<KEmoticonsProvider>(0);
+    KEmoticonsProvider *provider = factory->create<KEmoticonsProvider>(nullptr);
     return provider;
 }
 
 void KEmoticonsPrivate::changeTheme(const QString &path)
 {
-    QFileInfo info(path);
-    QString name = info.dir().dirName();
+    const QFileInfo info(path);
+    const QString name = info.dir().dirName();
 
     if (m_themes.contains(name)) {
         loadTheme(name);
@@ -95,18 +118,38 @@ void KEmoticonsPrivate::changeTheme(const QString &path)
 
 KEmoticonsTheme KEmoticonsPrivate::loadTheme(const QString &name)
 {
-    const int numberOfTheme = m_loaded.size();
-    for (int i = 0; i < numberOfTheme; ++i) {
-        const QString fName = m_loaded.at(i)->property(QStringLiteral("X-KDE-EmoticonsFileName")).toString();
-        const QString path = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "emoticons/" + name + '/' + fName);
+    const auto registerProvider = [this](const QString &name, const QString &path, KEmoticonsProvider *provider) {
+        if (m_preferredSize.isValid()) {
+            provider->setPreferredEmoticonSize(m_preferredSize);
+        }
+        KEmoticonsTheme theme(provider);
+        provider->loadTheme(path);
+        m_themes.insert(name, theme);
+        m_fileWatcher.addPath(path);
+        return theme;
+    };
+
+    for (const KPluginMetaData &plugin : qAsConst(m_plugins)) {
+        const QString fName = plugin.rawData().value(QStringLiteral("X-KDE-EmoticonsFileName")).toString();
+        const QString path = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("emoticons/") + name + QLatin1Char('/') + fName);
 
         if (QFile::exists(path)) {
-            KEmoticonsProvider *provider = loadProvider(m_loaded.at(i));
-            KEmoticonsTheme theme(provider);
-            provider->loadTheme(path);
-            m_themes.insert(name, theme);
-            m_fileWatcher.addPath(path);
-            return theme;
+            KEmoticonsProvider *provider = loadProvider(plugin);
+            if (provider) {
+                return registerProvider(name, path, provider);
+            }
+        }
+    }
+    // KF6: remove support for old plugins
+    for (const KService::Ptr &service : qAsConst(m_oldStylePlugins)) {
+        const QString fName = service->property(QStringLiteral("X-KDE-EmoticonsFileName")).toString();
+        const QString path = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("emoticons/") + name + QLatin1Char('/') + fName);
+
+        if (QFile::exists(path)) {
+            KEmoticonsProvider *provider = loadProvider(service);
+            if (provider) {
+                return registerProvider(name, path, provider);
+            }
         }
     }
     return KEmoticonsTheme();
@@ -130,8 +173,9 @@ KEmoticonsTheme KEmoticons::theme() const
 
 KEmoticonsTheme KEmoticons::theme(const QString &name) const
 {
-    if (d->m_themes.contains(name)) {
-        return d->m_themes.value(name);
+    KEmoticonsTheme theme = d->m_themes.value(name);
+    if (!theme.isNull()) {
+        return theme;
     }
 
     return d->loadTheme(name);
@@ -139,15 +183,14 @@ KEmoticonsTheme KEmoticons::theme(const QString &name) const
 
 QString KEmoticons::currentThemeName()
 {
-    KConfigGroup config(KSharedConfig::openConfig(QStringLiteral("kdeglobals")), "Emoticons");
-    QString name = config.readEntry("emoticonsTheme", "Breeze");
-    return name;
+    return s_global()->m_themeName;
 }
 
 QStringList KEmoticons::themeList()
 {
-    QStringList ls;
     const QStringList themeDirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("emoticons"), QStandardPaths::LocateDirectory);
+    QStringList ls;
+    ls.reserve(themeDirs.count());
 
     for (int i = 0; i < themeDirs.count(); ++i) {
         QDir themeQDir(themeDirs[i]);
@@ -165,30 +208,41 @@ void KEmoticons::setTheme(const KEmoticonsTheme &theme)
 
 void KEmoticons::setTheme(const QString &theme)
 {
-    KConfigGroup config(KSharedConfig::openConfig(QStringLiteral("kdeglobals")), "Emoticons");
-    config.writeEntry("emoticonsTheme", theme);
-    config.sync();
+    s_global()->setThemeName(theme);
+}
+
+KEmoticonsTheme::ParseMode KEmoticons::parseMode()
+{
+    return s_global()->m_parseMode;
+}
+
+void KEmoticons::setParseMode(KEmoticonsTheme::ParseMode mode)
+{
+    s_global()->setParseMode(mode);
 }
 
 KEmoticonsTheme KEmoticons::newTheme(const QString &name, const KService::Ptr &service)
 {
     KEmoticonsProvider *provider = d->loadProvider(service);
-    KEmoticonsTheme theme(provider);
-    theme.setThemeName(name);
+    if (provider) {
+        KEmoticonsTheme theme(provider);
+        theme.setThemeName(name);
 
-    provider->newTheme();
+        provider->newTheme();
 
-    return theme;
+        return theme;
+    }
+    return KEmoticonsTheme();
 }
 
 QStringList KEmoticons::installTheme(const QString &archiveName)
 {
     QStringList foundThemes;
-    KArchiveEntry *currentEntry = 0L;
-    KArchiveDirectory *currentDir = 0L;
-    KArchive *archive = 0L;
+    KArchiveEntry *currentEntry = nullptr;
+    KArchiveDirectory *currentDir = nullptr;
+    KArchive *archive = nullptr;
 
-    QString localThemesDir(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + "/emoticons");
+    const QString localThemesDir(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/emoticons"));
 
     if (localThemesDir.isEmpty()) {
         qCritical() << "Could not find a suitable place in which to install the emoticon theme";
@@ -233,10 +287,15 @@ QStringList KEmoticons::installTheme(const QString &archiveName)
         if (currentEntry->isDirectory()) {
             currentDir = dynamic_cast<KArchiveDirectory *>(currentEntry);
 
-            for (int i = 0; i < d->m_loaded.size(); ++i) {
-                QString fName = d->m_loaded.at(i)->property(QStringLiteral("X-KDE-EmoticonsFileName")).toString();
-
-                if (currentDir && currentDir->entry(fName) != NULL) {
+            for (const KPluginMetaData &plugin : qAsConst(d->m_plugins)) {
+                const QString fName = plugin.rawData().value(QStringLiteral("X-KDE-EmoticonsFileName")).toString();
+                if (currentDir && currentDir->entry(fName) != nullptr) {
+                    foundThemes.append(currentDir->name());
+                }
+            }
+            for (const KService::Ptr &service : qAsConst(d->m_oldStylePlugins)) {
+                const QString fName = service->property(QStringLiteral("X-KDE-EmoticonsFileName")).toString();
+                if (currentDir && currentDir->entry(fName) != nullptr) {
                     foundThemes.append(currentDir->name());
                 }
             }
@@ -254,7 +313,7 @@ QStringList KEmoticons::installTheme(const QString &archiveName)
         const QString &theme = foundThemes[themeIndex];
 
         currentEntry = const_cast<KArchiveEntry *>(rootDir->entry(theme));
-        if (currentEntry == 0) {
+        if (currentEntry == nullptr) {
             // qDebug() << "couldn't get next archive entry";
             continue;
         }
@@ -262,7 +321,7 @@ QStringList KEmoticons::installTheme(const QString &archiveName)
         if (currentEntry->isDirectory()) {
             currentDir = dynamic_cast<KArchiveDirectory *>(currentEntry);
 
-            if (currentDir == 0) {
+            if (currentDir == nullptr) {
                 // qDebug() << "couldn't cast archive entry to KArchiveDirectory";
                 continue;
             }
@@ -277,18 +336,14 @@ QStringList KEmoticons::installTheme(const QString &archiveName)
     return foundThemes;
 }
 
-void KEmoticons::setParseMode(KEmoticonsTheme::ParseMode mode)
+void KEmoticons::setPreferredEmoticonSize(const QSize &size)
 {
-    KConfigGroup config(KSharedConfig::openConfig(QStringLiteral("kdeglobals")), "Emoticons");
-    config.writeEntry("parseMode", int(mode));
-    config.sync();
+    d->m_preferredSize = size;
 }
 
-KEmoticonsTheme::ParseMode KEmoticons::parseMode()
+QSize KEmoticons::preferredEmoticonSize() const
 {
-    KConfigGroup config(KSharedConfig::openConfig(QStringLiteral("kdeglobals")), "Emoticons");
-    return (KEmoticonsTheme::ParseMode) config.readEntry("parseMode", int(KEmoticonsTheme::RelaxedParse));
+    return d->m_preferredSize;
 }
 
 #include "moc_kemoticons.cpp"
-
